//  InfiniteCanvasApplet.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.lang.Math;
import java.awt.event.*;
import java.net.*;
import java.awt.Cursor;
import java.awt.image.*;

public class InfiniteCanvasApplet extends JApplet implements ActionListener, Runnable {
	// general
	private OCVNavigationPanel navigationPanel = null;
	private int edge = 1;
	private OCVDocument document = null;
	private Color backgroundColor = Color.white;
	
	// controlling the canvas
	private OCVCanvas canvas = null;
	private boolean pause = true;
	private Thread animateThread = null;
	
	// navigation frames
	private boolean animationForward = true;
	
	// free navigation
	private Point2D.Double mousePosition;
	private double mouseOffsetX = 0;
	private double mouseOffsetY = 0;
	private int buttonPressed = 0;
	private int freeNavigationType = 0;
	JButton navigationBarToolButton = null;
	
	//Cursors
	private Cursor defaultCursor = null;
	private Cursor linkCursor = null;
	private Cursor cursorOpenHand = null;
	private Cursor cursorClosedHand = null;
	private Cursor cursorQT0 = null;
	private Cursor cursorQT45 = null;
	private Cursor cursorQT90 = null;
	private Cursor cursorQT135 = null;
	private Cursor cursorQT180 = null;
	private Cursor cursorQT225 = null;
	private Cursor cursorQT270 = null;
	private Cursor cursorQT315 = null;
	private Cursor cursorQTDefault = null;
	private ImageIcon buttonImageQT = null;
	private ImageIcon buttonImageDefault = null;
	private int freeNavigationCursorRotation = -1;

	
	
	// ----------------------------------------------------------------------
	// Init Methods
	// ----------------------------------------------------------------------
	public void init() { // init the applet		
		edge = getSize().width;
		
		//Execute a job on the event-dispatching thread: creating this applet's GUI.
		try {
			javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					createGUI(0);
				}
			});
		} catch (Exception e) { 
			System.err.println("createGUI didn't successfully complete");
		}
		// start loading the media
		// not earlier to make sure the ui is loaded
		new OCVLoader(this);
		
	}
	
	private void createGUI(int type) {
		if (type == 0) { // applet is starting
			// loading screen color
			String r,g,b;
			r = getParameter("R");
			g = getParameter("G");
			b = getParameter("B");
			if ((r!=null) && (g!=null) && (b!=null))
				backgroundColor = new Color((new Integer(r)).intValue(), 
											(new Integer(g)).intValue(), 
											(new Integer(b)).intValue()); 
			
			
			// reset default layoutmanager
			Container contentPane = getContentPane();
			contentPane.setLayout(null); // no grid
			
			// create navigation panel
			navigationPanel = new OCVNavigationPanel();
			navigationPanel.setLayout(null);
			navigationPanel.setSize(edge,40);
			navigationPanel.setLocation(0,edge);
			navigationPanel.setBorderColor(Color.black);
			navigationPanel.setBackground(backgroundColor);
			contentPane.add(navigationPanel);
			
			// add a progressbar to the panel
			JProgressBar progressBar = new JProgressBar(1, 10);
			progressBar.setSize(edge-10,30);
			progressBar.setLocation(5,5);
			progressBar.setValue(1); // start value
			progressBar.setStringPainted(true);
			progressBar.setString("Loading Document [1/12]");
			progressBar.setBackground(backgroundColor);
			navigationPanel.add(progressBar);
			
			// SplashImage 4 canvas 
			String path = getParameter("SplashImage");
			Image splashImage = null;
			if (path != null) {
				try {
					splashImage = getImage(new URL(getDocumentBase(),path));
				} catch (MalformedURLException e) {
					System.err.println("Couldn't load SplashImage!");
				}
			}
			
			// create canvas
			canvas = new OCVCanvas(edge, splashImage, backgroundColor);
			canvas.setSize(edge,edge);
			canvas.setLocation(0,0);
			canvas.setOpaque(true);
			contentPane.add(canvas);
			
			// create default cursors
			defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
			linkCursor = new Cursor(Cursor.HAND_CURSOR);

		} else { // finsihed loading -> add new elements to navigationPanel
			navigationPanel.removeAll(); //remove progressbar
			navigationPanel.setLayout(null);
			
			String navigationBarParameter = getParameter("NavigationButtons");
			if ((navigationBarParameter != null) && (navigationBarParameter.equals("NO"))) {
				navigationPanel.drawSeparatorLine(false);
				document.setNavigationButtons(false);
			}
					
			if (document.navigationTool() == 0) {// free navigation
				// Canvas
				canvas.addMouseListener(new OCVFreeNavigationMouseListener());
				canvas.addMouseMotionListener(new OCVFreeNavigationMouseMotionListener());
				addKeyListener(new OCVFreeNavigationKeyListener());
				
				
				// load cursors
				Image cursorImageOpenHand = null;
				Image cursorImageClosedHand = null;					
				Point point = new Point(10,10);
				
				try {
					cursorImageClosedHand = getImage(getClass().getResource("closed_hand.gif"));
					cursorImageOpenHand = getImage(getClass().getResource("open_hand.gif"));
					cursorOpenHand = getToolkit().createCustomCursor(cursorImageOpenHand,point,"cursorOpenHand");
					cursorClosedHand = getToolkit().createCustomCursor(cursorImageClosedHand,point,"cursorClosedHand");
					
					canvas.setCursor(cursorOpenHand); // first nav mode is default navigation
				} catch (Exception e) {
					System.err.println("Couldn't load arrow images!");
				}							
				cursorQTDefault = getToolkit().createCustomCursor(getMouseCursor(-1),point,"qtCursorDefault");
				cursorQT0 = getToolkit().createCustomCursor(getMouseCursor(0),point,"qtCursor0");
				cursorQT45 = getToolkit().createCustomCursor(getMouseCursor(45),point,"qtCursor45");
				cursorQT90 = getToolkit().createCustomCursor(getMouseCursor(90),point,"qtCursor90");
				cursorQT135 = getToolkit().createCustomCursor(getMouseCursor(135),point,"qtCursor135");
				cursorQT180 = getToolkit().createCustomCursor(getMouseCursor(180),point,"qtCursor180");
				cursorQT225 = getToolkit().createCustomCursor(getMouseCursor(225),point,"qtCursor225");
				cursorQT270 = getToolkit().createCustomCursor(getMouseCursor(270),point,"qtCursor270");
				cursorQT315 = getToolkit().createCustomCursor(getMouseCursor(315),point,"qtCursor315");
				
				
				if (document.navigationButtons()) { 					
					// Home Button
					JButton home = new JButton(new OCVButtonIcons(4));
					OCVFreeNavigationButtonPressedListener adapter = new OCVFreeNavigationButtonPressedListener();
					adapter.type = 5;
					home.addMouseListener(adapter);
					home.setSize(30,30);
					home.setLocation(edge/2-1-30-10-30-2-30,5);
					home.setVerticalTextPosition(AbstractButton.CENTER);
					home.setHorizontalTextPosition(AbstractButton.CENTER);
					home.setBackground(backgroundColor);
					home.setMargin(new Insets(0,0,0,0));		
					home.setRequestFocusEnabled(false);
					navigationPanel.add(home);
					
					// Navigation Tool Button
					buttonImageDefault = new ImageIcon(cursorImageOpenHand.getScaledInstance(20,20,Image.SCALE_SMOOTH));
					buttonImageQT = OCVButtonIcons.getFreeNavigationImageIcon();
					navigationBarToolButton = new JButton(buttonImageQT);
					adapter = new OCVFreeNavigationButtonPressedListener();
					adapter.type = 6;
					navigationBarToolButton.addMouseListener(adapter);
					navigationBarToolButton.setSize(30,30);
					navigationBarToolButton.setLocation(edge/2-1-30-10-30,5);
					navigationBarToolButton.setVerticalTextPosition(AbstractButton.CENTER);
					navigationBarToolButton.setHorizontalTextPosition(AbstractButton.CENTER);
					navigationBarToolButton.setBackground(backgroundColor);
					navigationBarToolButton.setMargin(new Insets(0,0,0,0));	
					navigationBarToolButton.setRequestFocusEnabled(false);
					navigationPanel.add(navigationBarToolButton);
					
					
					// Zoom In Button
					JButton zoomIn = new JButton(new OCVButtonIcons(1));
					adapter = new OCVFreeNavigationButtonPressedListener();
					adapter.type = 2;
					zoomIn.addMouseListener(adapter);
					zoomIn.setSize(30,30);
					zoomIn.setLocation(edge/2-1-30,5);
					zoomIn.setVerticalTextPosition(AbstractButton.CENTER);
					zoomIn.setHorizontalTextPosition(AbstractButton.CENTER);
					zoomIn.setBackground(backgroundColor);
					zoomIn.setMargin(new Insets(0,0,0,0));		
					zoomIn.setRequestFocusEnabled(false);
					navigationPanel.add(zoomIn);
					
					// Zoom Out Button
					JButton zoomOut = new JButton(new OCVButtonIcons(0));
					adapter = new OCVFreeNavigationButtonPressedListener();
					adapter.type = 1;
					zoomOut.addMouseListener(adapter);
					zoomOut.setSize(30,30);
					zoomOut.setLocation(edge/2+1,5);
					zoomOut.setVerticalTextPosition(AbstractButton.CENTER);
					zoomOut.setHorizontalTextPosition(AbstractButton.CENTER);
					zoomOut.setBackground(backgroundColor);
					zoomOut.setMargin(new Insets(0,0,0,0));	
					zoomOut.setRequestFocusEnabled(false);
					navigationPanel.add(zoomOut);
					
					// Rotate Counter Clockwise
					JButton rotateCCW = new JButton(new OCVButtonIcons(2));
					adapter = new OCVFreeNavigationButtonPressedListener();
					adapter.type = 4;
					rotateCCW.addMouseListener(adapter);
					rotateCCW.setName("CCW");
					rotateCCW.setSize(30,30);
					rotateCCW.setLocation(edge/2+1+30+10,5);
					rotateCCW.setVerticalTextPosition(AbstractButton.CENTER);
					rotateCCW.setHorizontalTextPosition(AbstractButton.CENTER);
					rotateCCW.setBackground(backgroundColor);
					rotateCCW.setMargin(new Insets(0,0,0,0));
					rotateCCW.setRequestFocusEnabled(false);
					navigationPanel.add(rotateCCW);
					
					// Rotate Counter Clockwise
					JButton rotateCW = new JButton(new OCVButtonIcons(3));
					adapter = new OCVFreeNavigationButtonPressedListener();
					adapter.type = 3;
					rotateCW.addMouseListener(adapter);
					rotateCW.setSize(30,30);
					rotateCW.setLocation(edge/2+1+30+10+30+2,5);
					rotateCW.setVerticalTextPosition(AbstractButton.CENTER);
					rotateCW.setHorizontalTextPosition(AbstractButton.CENTER);
					rotateCW.setBackground(backgroundColor);
					rotateCW.setMargin(new Insets(0,0,0,0));	
					rotateCW.setRequestFocusEnabled(false);
					navigationPanel.add(rotateCW);
				}
			} else { // navigation frames
				// Canvas
				canvas.addMouseListener(new OCVNavigationFramesMouseListener());
				canvas.addMouseMotionListener(new OCVNavigationFramesMouseMotionListener());
				if (document.navigationButtons()) { 
					// Back Button
					JButton back = new JButton(new OCVArrowIcon(SwingConstants.LEADING));
					back.setName("back");
					back.addActionListener(this);
					back.setSize(30,30);
					back.setLocation(5,5);
					back.setVerticalTextPosition(AbstractButton.CENTER);
					back.setHorizontalTextPosition(AbstractButton.CENTER);
					back.setBackground(backgroundColor);
					back.setMargin(new Insets(0,0,0,0));
					navigationPanel.add(back);
					
					// Next Button
					JButton next = new JButton(new OCVArrowIcon(SwingConstants.TRAILING));
					next.setName("next");
					next.addActionListener(this);
					next.setSize(30,30);
					next.setLocation(edge-30-5,5);
					next.setVerticalTextPosition(AbstractButton.CENTER);
					next.setHorizontalTextPosition(AbstractButton.CENTER);
					next.setBackground(backgroundColor);
					next.setMargin(new Insets(0,0,0,0));
					navigationPanel.add(next);
				}
			}
			navigationPanel.validate();
			navigationPanel.repaint();
		}
	}
	
	
	private Image getMouseCursor(int type) {
		BufferedImage image =  new BufferedImage(34,34,BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = (Graphics2D) image.createGraphics();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setStroke(new BasicStroke(1)); // set thickness = 1
		g2.setColor(Color.white);
		g2.fillOval(11,11,12,12);
		g2.setColor(Color.black);
		g2.fillOval(15,15,4,4);
		g2.drawOval(13,13,8,8);

		if (type != -1) {
			double theta = OCVMath.toRadians(type); 
			g2.rotate(-theta, 17, 17);
			
			int[] xPoints = new int[3];
			int[] yPoints = new int[3];
			
			xPoints[0] = 25;
			yPoints[0] = 9;
			xPoints[1] = 34;
			yPoints[1] = 17;
			xPoints[2] = 25;
			yPoints[2] = 25;
			g2.setColor(Color.white);
			g2.fillPolygon(xPoints, yPoints, 3);

			xPoints[0] = 27;
			yPoints[0] = 12;
			xPoints[1] = 31;
			yPoints[1] = 17;
			xPoints[2] = 27;
			yPoints[2] = 22;
			g2.setColor(Color.black);
			g2.fillPolygon(xPoints, yPoints, 3);
			
			g2.rotate(+theta, -17, -17);   //Restore graphics object
		} 
		
		return (Image) image;
	}

	
	public void setLoadingStatus (int status) {
		if (navigationPanel != null) {
			JProgressBar progressBar = (JProgressBar) navigationPanel.getComponent(0);
			if (progressBar != null) {
				progressBar.setValue(status);
				if (status == 1) {
					progressBar.setString("Loading Document [1/10]");	
				} else if (status < 0) {
					progressBar.setString("Loading Failed (Try reloading the webpage!). ErrorCode: "+status);	
				} else {
					progressBar.setString("Loading Panels ["+status+"/10]");	
				}
			}
		}
	}
	
	public void startDisplayingDocument (OCVDocument aDocument) {
		document = aDocument;
		
		if (canvas.startDisplayingDocument(document)) {
			try {
				javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
					public void run() {
						createGUI(1);
					}
				});
			} catch (Exception e) { 
				System.err.println("createGUI didn't successfully complete");
			}
			validateNavigationButtons(true);
		} else {
			setLoadingStatus(-5);
			while (true) {
					// infinite loop -> don't do anything
			}
		}
		
	}	
	
	 
	// ----------------------------------------------------------------------
	// Applet Methods
	// ----------------------------------------------------------------------
    public void start () {
        pause = false;
    }
    
    public void stop () {
        pause = true;
    }
    
   /* public void destroy () {
    }*/
	
	public String[][] getParameterInfo() {
        String[][] info = {{"Document",	"String", "an Infinite Canvas Document"},
		{"SplashImage", "String",	"Path to the Splash Image"},
		{"NavigationPanel", "Boolean","Display Navigation Panel - use 'true' or 'false'"}};
		return info;
    }    
	
    public String getAppletInfo() {
        return "Infinite Canvas Applet (Version 1.1) by Markus Mueller";
    }
	
	// ----------------------------------------------------------------------
	// Convenient Methods
	// ----------------------------------------------------------------------
	protected static ImageIcon createImageIcon(String path, String description) {
		java.net.URL imgURL = InfiniteCanvasApplet.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL, description);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
	
	// ----------------------------------------------------------------------
	// Button Input Methods
	// ----------------------------------------------------------------------
	public void actionPerformed(ActionEvent event) {
		if (document.navigationTool() == 1) {
			String objectName = ((Component)event.getSource()).getName();
			if (objectName.equals("next")) {
				initAnimationForward(true);
			} else{ // Back
				initAnimationForward(false);
			}
		}
	}
	
	class OCVFreeNavigationButtonPressedListener extends MouseAdapter {
		public int type = 0;
		
        public void mousePressed(MouseEvent event) {
			buttonPressed = type;
			if (type == 1) { // ZoomOut
				canvas.zoomOut();
				buttonPressedTimer.start();
			} else if (type == 2) { //ZoomIN
				canvas.zoomIn();
				buttonPressedTimer.start();
			} else if (type == 3) { // CW
				canvas.rotateCW();				
				buttonPressedTimer.start();
			} else if (type == 4) { // CCW
				canvas.rotateCCW();
				buttonPressedTimer.start();
			} else if (type == 5) {
				canvas.resetFreeNavigation();
			} else if (type == 6) {
				buttonPressed = 0;			// just for ...
				buttonPressedTimer.stop();  // ... safty

				if (freeNavigationType == 0) {
					freeNavigationType = 1;
					freeNavigationCursorRotation = 0; // work-around to make sure that cursor -1 will be set
					validateQTCursor();
					navigationBarToolButton.setIcon(buttonImageDefault);
				} else {
					freeNavigationType = 0;	
					canvas.setCursor(cursorOpenHand);
					navigationBarToolButton.setIcon(buttonImageQT);
				}
			}
        }
		
        public void mouseReleased(MouseEvent event) {
            buttonPressed = 0;
            buttonPressedTimer.stop();
        }
		
        public void mouseExited(MouseEvent event) {
            buttonPressed = 0;
            buttonPressedTimer.stop();
        }
    };
	
    private javax.swing.Timer buttonPressedTimer = new javax.swing.Timer(100, new ActionListener(){
        public void actionPerformed(ActionEvent event) {
            if (buttonPressed == 1) { // ZoomOut
				canvas.zoomOut();
			} else if (buttonPressed == 2) { //ZoomIN
				canvas.zoomIn();
			} else if (buttonPressed == 3) { // CW
				canvas.rotateCW();
			} else if (buttonPressed == 4) { // CCW
				canvas.rotateCCW();
			}				
        }
    });	
	
	
	
	private void validateNavigationButtons(boolean validate) {
		if ((document.navigationTool() == 1) && (document.navigationButtons())){ // only in navigation frames mode
			if (validate) {
				if (canvas.validPreviousFrame()) {
					navigationPanel.getComponent(0).setEnabled(true);
				} else {
					navigationPanel.getComponent(0).setEnabled(false);
				}
				if (canvas.validNextFrame()) {
					navigationPanel.getComponent(1).setEnabled(true);
				} else {
					navigationPanel.getComponent(1).setEnabled(false);
				}			
			} else {
				navigationPanel.getComponent(0).setEnabled(false);
				navigationPanel.getComponent(1).setEnabled(false);
			}
		}
	}
	
	// ----------------------------------------------------------------------
	// Animation Methods
	// ----------------------------------------------------------------------
	
	private void initAnimationForward(boolean forward) {
		animationForward = forward; // needed in run
		
		// disable buttons and set default cursor
		validateNavigationButtons(false);
		canvas.setCursor(defaultCursor); 
		
		// calculate everything
		int status = canvas.calculateAnimationForward(forward);
		if (status == 0) { // error
			validateNavigationButtons(true);
		} else if (status == 1) { // no need for animation
			finishAnimationForward(forward);
		} else {
			startAnimationForward(forward);
		}
	}
	
	private void startAnimationForward(boolean forward) {
		animateThread = new Thread(this);
		animateThread.start();    
	}
	
	private void finishAnimationForward(boolean forward) {
		animateThread = null;
		canvas.finishAnimation();
		canvas.centerActiveFrame(); // center active frame
		if (!canvas.waitForInput()) // start Animation without waiting for userinput?
			initAnimationForward(forward);
		else
			validateNavigationButtons(true);
	}
	
	public void run() {
		if (animateThread == Thread.currentThread()) {
			long time = System.currentTimeMillis(); 
			long interval = 30;
			
			while (canvas.stepAnimation(animationForward)) {
				while (pause) { // check every second if still in pause mode
					time += 100;
					try {
						Thread.sleep(Math.max(0, time - System.currentTimeMillis()));
					} catch (InterruptedException e) {
						break;
					}
				}
				canvas.repaint();
				try {
					time += interval;
					Thread.sleep(Math.max(0, time - System.currentTimeMillis()));
				} catch (InterruptedException e) {
					break;
				}
			}
			
			finishAnimationForward(animationForward);		
		}
	}
	
	// ----------------------------------------------------------------------
	// Canvas Input Methods - Navigation Frames
	// ----------------------------------------------------------------------
	class OCVNavigationFramesMouseListener extends MouseAdapter {
		
		public void mousePressed(MouseEvent event) {
			if (animateThread == null) {
				Point point = event.getPoint();
				
				double factor = canvas.scaleFactor();
				point.x *= factor;
				point.y *= factor;
				
				canvas.activeComponentHit(point, true);
				initAnimationForward(true);
				
			}
		}
	}
	
	class OCVNavigationFramesMouseMotionListener extends MouseMotionAdapter {

		public void mouseMoved(MouseEvent event) {
			if (animateThread == null) {
				Point point = event.getPoint();
				
				double factor = canvas.scaleFactor();
				point.x *= factor;
				point.y *= factor;

				if( (canvas.activeComponentHit(point,false)) && (document.activeComponentHint()) ) {
					canvas.setCursor(linkCursor); 
				} else {
					canvas.setCursor(defaultCursor); 
				}
			}
		}
		
	}
	
	// ----------------------------------------------------------------------
	// Canvas Input Methods - Free Navigation
	// ----------------------------------------------------------------------
	class OCVFreeNavigationMouseListener extends MouseAdapter {
		
		public void mousePressed(MouseEvent event) {
			double factor = canvas.scaleFactor();
			mousePosition = new Point2D.Double(event.getX(),event.getY());
					
			if (freeNavigationType == 0) { // default
				canvas.setCursor(cursorClosedHand);
			} else { // like QT
				mouseOffsetX = 0;
				mouseOffsetY = 0;
				freeNavigationTimer.start();
			}
		}
		
		public void mouseReleased(MouseEvent event) {
			if (freeNavigationType == 0) { // default
				canvas.setCursor(cursorOpenHand);
			} else { // like QT
				freeNavigationTimer.stop();
				mouseOffsetX = 0;
				mouseOffsetY = 0;
				validateQTCursor();
			}
		}
	}
	
	class OCVFreeNavigationMouseMotionListener extends MouseMotionAdapter {
		
		public void mouseDragged(MouseEvent event) {
			double factor = canvas.scaleFactor();
			int x = event.getX();
			int y = event.getY();
			
			if (freeNavigationType == 0) { // default
				canvas.moveXY((mousePosition.x-x) * factor, (mousePosition.y-y) * factor);
				mousePosition = new Point2D.Double(x,y);
			} else { // like QT
				mouseOffsetX = (x - mousePosition.x) * factor / 8;
				/*double mouseAbs = Math.abs(mouseOffsetX);
				if (mouseAbs < 1) { // don't move if cursor is near center
					mouseOffsetX = 0;
				} */
				
				mouseOffsetY = (y - mousePosition.y) * factor / 8;
				/*mouseAbs = Math.abs(mouseOffsetY);
				if (mouseAbs < 1) {
					mouseOffsetY = 0;
				}*/ 
				validateQTCursor();
			}
		}		
	}
	
	private javax.swing.Timer freeNavigationTimer = new javax.swing.Timer(30, new ActionListener(){
        public void actionPerformed(ActionEvent event) {
			canvas.moveXY(mouseOffsetX,mouseOffsetY); 
        }
    });	
	
	class OCVFreeNavigationKeyListener extends KeyAdapter {
        public void keyPressed(KeyEvent event) {
            if (event.getKeyCode() == KeyEvent.VK_UP)  {
				canvas.moveXY(0,-10);
            } else if (event.getKeyCode() == KeyEvent.VK_DOWN) {
				canvas.moveXY(0,10);
            } else if (event.getKeyCode() == KeyEvent.VK_LEFT) {
				canvas.moveXY(-10,0);
            } else if (event.getKeyCode() == KeyEvent.VK_RIGHT) {
				canvas.moveXY(10,0);
            } 
		}
    }
	
	
	private void validateQTCursor() {
		int radius = -1;
		double diff = Math.abs(mouseOffsetX/mouseOffsetY);

		if ((mouseOffsetX == 0) && (mouseOffsetY == 0)) {
			radius = -1;
		} else if ((mouseOffsetX >= 0) && (mouseOffsetY >= 0)) {
			if ((diff < 2) && (diff > 0.5)) {
				radius = 315;
			} else if (mouseOffsetX > mouseOffsetY) {
				radius = 0;
			} else {
				radius = 270;
			}
		} else if ((mouseOffsetX <= 0) && (mouseOffsetY >= 0)){
			if ((diff < 2) && (diff > 0.5)) {
				radius = 225;
			} else if ((mouseOffsetX * -1) > mouseOffsetY) {
				radius = 180;
			} else {
				radius = 270;
			}			
		} else if ((mouseOffsetX <= 0) && (mouseOffsetY <= 0)){
			if ((diff < 2) && (diff > 0.5)) {
				radius = 135;
			} else if ((mouseOffsetX * -1) > (mouseOffsetY * -1)) {
				radius = 180;
			} else {
				radius = 90;
			}			
		} else {
			if ((diff < 2) && (diff > 0.5)) {
				radius = 45;
			} else if (mouseOffsetX > (mouseOffsetY * -1)) {
				radius = 0;
			} else {
				radius = 90;
			}			
		}
		if (freeNavigationCursorRotation != radius) {
			switch (radius) {
					case 0:
						canvas.setCursor(cursorQT0);
						break;
					case 45:
						canvas.setCursor(cursorQT45);
						break;
					case 90:
						canvas.setCursor(cursorQT90);
						break;
					case 135:
						canvas.setCursor(cursorQT135);
						break;
					case 180:
						canvas.setCursor(cursorQT180);
						break;
					case 225:
						canvas.setCursor(cursorQT225);
						break;
					case 270:
						canvas.setCursor(cursorQT270);
						break;
					case 315:
						canvas.setCursor(cursorQT315);
						break;			
					default: // -1
						canvas.setCursor(cursorQTDefault);
			}
			freeNavigationCursorRotation = radius;
		}
	}
	
}
