//
//  OCENavLineObject.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCENavTrailObject.h"
#import "OCERectObject.h"
#import "OCENavFrameObject.h"
#import "OCEActiveComponent.h"

@implementation OCENavTrailObject

- (id)init 
{
    [super init];
	[self setColor: [NSColor blueColor]];
	allowFreeEndPoints = NO;
	[self setSnapAble:YES];
	transition = 2;
	transitionSpeed = 3;
	
    return self;
}

- (bool) valid
{
	int count = [path elementCount];
	if (count > 1 && startRect && endRect) {
		if (!((count == 2) && (startRect == endRect))) {
			return YES;
		}
	} 
	return NO;
}

- (void) adaptPathToRects
// funktioniert etwas anders als die geerbte da wir nicht an der mitte clippen
{
	if (startRect)
		[self setStartPoint:[startRect center]];
	
	if (endRect)
		[self setEndPoint:[endRect center]];
}

- (void) doubleKlickAtPoint: (NSPoint) aPoint
{
	// super berschrieben, weil wir hier keine kurven wollen!!
	
}

- (void) setAssociatedStartRect: (OCERectObject *) rectObject
{
	[super setAssociatedStartRect: rectObject];
	[self setColor:[(OCENavFrameObject *)rectObject borderColor]];
	// beim wechsel des Star Nav Frames soll der Nav Trail auch die Farbe des neuen NavTrails bekommen
}

- (int) transition
{
	return transition;
}

- (void) setTransition: (int) flag
{
	if (transition != flag)
	{
		[[undoManager prepareWithInvocationTarget:self] setTransition:transition]; 
		[undoManager setActionName: @"Change Transition Type"];

		transition = flag;
	}
}

- (int) transitionSpeed
{
	return transitionSpeed;
}

- (void) setTransitionSpeed: (int) flag
{
	if (transitionSpeed != flag)
	{
		[[undoManager prepareWithInvocationTarget:self] setTransitionSpeed:transitionSpeed]; 
		[undoManager setActionName: @"Change Transition Speed"];

		transitionSpeed = flag;
	}
}

// Navigation
- (int) segments
{
	return [path elementCount]-1;
}

- (NSPoint) endPointOfSegment: (int) segment
{
	if ((segment < 0) || (segment >= [path elementCount]))
		return NSZeroPoint;
	
	NSPoint *points = malloc(3*sizeof(NSPoint));
	NSPoint returnPoint;
	NSBezierPathElement element = [path elementAtIndex: segment associatedPoints:points];
	
	if (element == NSCurveToBezierPathElement)
		returnPoint = points[2];
	else
		returnPoint = points[0];
	
	free(points);
	
	return returnPoint;
}

- (int) length
{
	int count = [path elementCount];
	int i = 1;
	int length = 0;
	
	while (i < count)
	{
		length += [self lengthOfSegment:i];
		i++;
	}
	return length;
}

- (int) lengthOfSegment: (int) segment
{
	NSPoint startPoint;
	if ((segment-1 == 0) && ([startRect isActiveComponent]))
		startPoint = [[(OCEActiveComponent *)startRect frame] center];
	else
		startPoint = [self endPointOfSegment: segment-1];
	
	NSPoint endPoint = [self endPointOfSegment: segment];
	float width = endPoint.x - startPoint.x;
	float height = endPoint.y - startPoint.y;

	return abs(sqrt(width*width+height*height)); //pythagoras
}

// ----------------------------------------------------------------------
// NSCopying Protocol
// ----------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
{
    OCENavTrailObject *copy = [super copyWithZone: zone];

    copy->transition = transition;
    copy->transitionSpeed = transitionSpeed;
	copy->startRect = startRect;
	copy->endRect = endRect;
    
    return copy;
}

// ----------------------------------------------------------------------
// Save and Load
// ----------------------------------------------------------------------
- (NSMutableArray *)dataRepresentation
{
	NSMutableArray *array = [[super dataRepresentation] retain];
	
	[array addObject:[NSNumber numberWithInt:transition]];
	[array addObject:[NSNumber numberWithInt:transitionSpeed]];

	return [array autorelease];
}

- (NSMutableString *) stringRepresentation {
	NSMutableString *data = [[super stringRepresentation] retain];
	
	[data appendFormat: @"<i>%i</i>\n", transition];
	[data appendFormat: @"<i>%i</i>\n", transitionSpeed];
	
	return [data autorelease];
}

- (id)initWithContentsOfArray:(NSArray *) array forDocument: (OCEDocument *) newDocument
{
    [self init];
    transition = [[array objectAtIndex:10] intValue];
    transitionSpeed = [[array objectAtIndex:11] intValue];
    return [super initWithContentsOfArray: array forDocument: newDocument];
}


@end
