//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 


import java.awt.*;
import javax.swing.*;
import java.awt.image.*;

public class OCVButtonIcons implements Icon {
    private int width = 10;
    private int height = 10;
	private int type = 0;
	
    private int[] xPoints = null;
    private int[] yPoints = null;
	
    public OCVButtonIcons(int newType) {	
		type = newType;
		int halfWidth = width/2;
		int halfHeight = height/2;
        if (type == 0) { // - 
			xPoints = new int[4];
			yPoints = new int[4];
            xPoints[0] = 0;
            yPoints[0] = halfHeight - 1;
            xPoints[1] = 0;
            yPoints[1] = halfHeight + 1;
            xPoints[2] = width;
            yPoints[2] = halfHeight + 1;
            xPoints[3] = width;
            yPoints[3] = halfHeight - 1;
        } else if (type == 1) { // +
			xPoints = new int[12];
			yPoints = new int[12];
            xPoints[0] = 0;
            yPoints[0] = halfHeight - 1;
            xPoints[1] = 0;
            yPoints[1] = halfHeight + 1;
            xPoints[2] = halfWidth-1;
            yPoints[2] = halfHeight + 1;
            xPoints[3] = halfWidth-1;
            yPoints[3] = height;
			xPoints[4] = halfWidth+1;
            yPoints[4] = height;
            xPoints[5] = halfWidth+1;
            yPoints[5] = halfHeight+1;
            xPoints[6] = width;
            yPoints[6] = halfHeight + 1;
            xPoints[7] = width;
            yPoints[7] = halfHeight-1;
			xPoints[8] = halfWidth+1;
            yPoints[8] = halfHeight-1;
            xPoints[9] = halfWidth+1;
            yPoints[9] = 0;
            xPoints[10] = halfWidth-1;
            yPoints[10] = 0;
            xPoints[11] = halfWidth-1;
            yPoints[11] = halfHeight-1;
        } else if (type == 2) { // rotate CCW
			xPoints = new int[3];
			yPoints = new int[3];
            xPoints[0] = 3;
            yPoints[0] = 10;
            xPoints[1] = 8;
            yPoints[1] = 10;
            xPoints[2] = 8;
            yPoints[2] = 5;
		} else if (type == 3) { // rotate CW
			xPoints = new int[3];
			yPoints = new int[3];
            xPoints[0] = 7;
            yPoints[0] = 10;
            xPoints[1] = 2;
            yPoints[1] = 10;
            xPoints[2] = 2;
            yPoints[2] = 5;
		} else if (type == 4) { // home
		}
    }
	
    public int getIconHeight() {
        return height;
    }
	
    public int getIconWidth() {
        return width;
    }
	
    public void paintIcon(Component c, Graphics g, int x, int y) {
		Graphics2D g2 = (Graphics2D) g; 

        if (c.isEnabled()) {
            g2.setColor(c.getForeground());
        } else {
            g2.setColor(Color.gray);
        }
		
        g2.translate(x, y);
		
		if ((type == 0) || (type == 1)) {
			g2.fillPolygon(xPoints, yPoints, xPoints.length);
		} else if (type == 2) {
			g2.fillPolygon(xPoints, yPoints, xPoints.length);
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setStroke(new BasicStroke(2));
			g2.drawArc(3, 0, 4, 7,  90,  160);
		} else if (type == 3) {
			g2.fillPolygon(xPoints, yPoints, xPoints.length);
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setStroke(new BasicStroke(2));
			g2.drawArc(3, 0, 4, 7,  90,  -160);
		} else if (type == 4) {
			drawHomeIcon(g2);
		} else if (type == 5) {
			
		}
        g.translate(-x, -y);   //Restore graphics object
    }
	
	private void drawHomeIcon(Graphics2D g2) {
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setStroke(new BasicStroke((float)2));

		// roof
		g2.drawLine(0,5,5,0);
		g2.drawLine(5,0,10,5);
		
		// wall
		g2.drawLine(2,4,2,10);
		g2.drawLine(2,10,8,10);
		g2.drawLine(8,4,8,10);
		g2.drawLine(2,4,8,4);
	}
	
	
	
	static ImageIcon getFreeNavigationImageIcon() {
		BufferedImage image =  new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = (Graphics2D) image.createGraphics();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setColor(Color.black);

		int[] xPoints = new int[3];
		int[] yPoints = new int[3];
		
		// right
		xPoints[0] = 14;
		yPoints[0] = 6;
		xPoints[1] = 18;
		yPoints[1] = 10;
		xPoints[2] = 14;
		yPoints[2] = 14;
		g2.fillPolygon(xPoints, yPoints, 3);

		// left
		xPoints[0] = 4;
		yPoints[0] = 6;
		xPoints[1] = 0;
		yPoints[1] = 10;
		xPoints[2] = 4;
		yPoints[2] = 14;
		g2.fillPolygon(xPoints, yPoints, 3);

		// up
		xPoints[0] = 5;
		yPoints[0] = 5;
		xPoints[1] = 9;
		yPoints[1] = 1;
		xPoints[2] = 13;
		yPoints[2] = 5;
		g2.fillPolygon(xPoints, yPoints, 3);
		
		// down
		xPoints[0] = 5;
		yPoints[0] = 15;
		xPoints[1] = 9;
		yPoints[1] = 19;
		xPoints[2] = 13;
		yPoints[2] = 15;
		g2.fillPolygon(xPoints, yPoints, 3);
		
		
		return new ImageIcon(image);
	}
	
}
