//
//  OCVDocument.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 


import java.awt.*;
import java.util.*;


public class OCVDocument {
    private Vector canvasObjects;
	private Vector navigationObjects;
    private String arrowImagePath;
    private Color canvasColor;
	private int navigationTool;
	private boolean navigationButtons;
	private boolean activeComponentHint;
	private Image arrowImage = null;
	private Image activeComponentHintImage = null;
	private Image nextImage = null;
	private Image backImage = null;

    public OCVDocument () {
        canvasObjects = new Vector();
		navigationObjects = new Vector();
		navigationTool=0;
		navigationButtons=false;
		activeComponentHint=false;
    }
	
	public void initWithVector(Vector vector) {
		// version check
		if (!((String) vector.elementAt(0)).equals("1.1-applet"))
			return;
		
		//arrowImagePath = (String) vector.elementAt(1);
		setCanvasColor(((Float)vector.elementAt(1)).floatValue(), ((Float)vector.elementAt(2)).floatValue(), 
				 ((Float)vector.elementAt(3)).floatValue());

		// Navigation	
		navigationTool = ((Integer)vector.elementAt(4)).intValue();
		navigationButtons = true; /*((Boolean)vector.elementAt(5)).booleanValue();*/ 
		activeComponentHint = ((Boolean)vector.elementAt(6)).booleanValue();
		
		
		// canvasObjects
		Vector objectsVector = ((Vector) vector.elementAt(7));
		int i = objectsVector.size();
		int c = i;
		
		for (i=0; i<c; i++)  {
			Vector object = (Vector)(objectsVector.elementAt(i));
			if (((String) object.elementAt(0)).equals("OCEPanelObject")) {
				OCVPanelObject panelObject = new OCVPanelObject();
				panelObject.initWithVector(object);
				canvasObjects.add(panelObject);
			} else {
				OCVTrailObject trailObject = new OCVTrailObject();
				trailObject.initWithVector(object);
				canvasObjects.add(trailObject);
			}
		}
		
		// navigationObjects
		objectsVector = ((Vector) vector.elementAt(8));
		i = objectsVector.size();
		c = i;
		
		for (i=0; i<c; i++) {
			Vector object = (Vector)(objectsVector.elementAt(i));
			if (((String) object.elementAt(0)).equals("OCENavFrameObject")) {
				OCVNavFrameObject frameObject = new OCVNavFrameObject();
				frameObject.initWithVector(object);
				navigationObjects.add(frameObject);
			} else {
				OCVNavTrailObject trailObject = new OCVNavTrailObject();
				trailObject.initWithVector(object);
				navigationObjects.add(trailObject);
			}
		}
	}
       
    public Vector canvasObjects() {
        return canvasObjects;
    }
    
    public void addCanvasObject(OCVObject object){
        canvasObjects.addElement(object);
    }
	
	public Vector navigationObjects() {
        return navigationObjects;
    }
    
    public void addNavigationObject(OCVObject object) {
        navigationObjects.addElement(object);
    }
    
    public Color canvasColor() {
        return canvasColor;
    }
    
    public void setCanvasColor(float r, float g, float b) {
        canvasColor = new Color(r,g,b);
    }
    
    public String arrowImagePath() {
        return arrowImagePath;
    }
    
    public void setArrowImagePath(String path) {
        arrowImagePath = path;
    }
	
	public void setArrowImage (Image image) {
		arrowImage = image;
	}
	
	public Image arrowImage () {
		return arrowImage;
	}
	
	public int navigationTool() {
		return navigationTool;
	}
	
	public void setNavigationTool(int tool) {
		navigationTool = tool;	
	}
	
	public boolean navigationButtons() {
		return navigationButtons;
	}
	
	public void setNavigationButtons(boolean flag) {
		navigationButtons = flag;	
	}
	
	public boolean activeComponentHint() {
		return activeComponentHint;
	}
	
	public void setActiveComponentHint(boolean flag) {
		activeComponentHint = flag;	
	}
	
	public OCVRectObject startFrame () {
        for (Enumeration e = navigationObjects.elements() ; e.hasMoreElements() ;) {
			OCVObject object = (OCVObject) e.nextElement();
            if (object.className().equals("OCVNavFrameObject"))
				if (((OCVNavFrameObject)object).isStartFrame())
					return (OCVRectObject) object;
        }
        return null;
    }
	
	public Vector outgoingPathsForFrame (OCVRectObject frame) {
		Vector outgoing = new Vector();
		for (Enumeration e = navigationObjects.elements() ; e.hasMoreElements() ;) {
			int rectID = frame.rectID();
			OCVObject object = (OCVObject) e.nextElement();
            if (object.className().equals("OCVNavTrailObject"))
				if (((OCVNavTrailObject)object).startRectID() == rectID)
					 outgoing.add((OCVNavTrailObject) object);
        }
        return outgoing;
	}
	
	public OCVRectObject rectObjectFromID(int id) {
        for (Enumeration e = navigationObjects.elements() ; e.hasMoreElements() ;) {
			OCVObject object = (OCVObject) e.nextElement();
            if (object.className().equals("OCVNavFrameObject")) {
				if (((OCVRectObject)object).rectID() == id) {
					return (OCVRectObject) object;
				} else {
					object = ((OCVNavFrameObject)object).activeComponentFromID(id);
					if (object != null)
						return (OCVRectObject) object;
				}
			}
        }
		return null;
	}
	
	public void setActiveComponentHintImage(Image image) {
		activeComponentHintImage = image;
	}
	
	public Image activeComponentHintImage() {
		return activeComponentHintImage;
	}
	
	public void setNextImage(Image image) {
		nextImage = image;
	}
	
	public Image nextImage() {
		return nextImage;	
	}

	public void setBackImage(Image image) {
		backImage = image;
	}
	
	public Image backImage() {
		return backImage;	
	}
	
	public void finishLoading(InfiniteCanvasApplet applet) {
        for (Enumeration e = canvasObjects.elements() ; e.hasMoreElements() ;) {
			OCVObject object = (OCVObject) e.nextElement();
			object.finishLoading(applet);
        }
		
	}
}