//
//  DocumentUnmarshaller.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 

import java.util.*;
import java.io.*;
import java.net.*;

public class OCVDocumentUnmarshaller  {
    private Vector vector;
    private Stack stack;
    private boolean isStackReadyForText;

    // Constructor
    public OCVDocumentUnmarshaller() {
        stack = new Stack();
        isStackReadyForText = false;
    }

    public Vector vector() { 
        return vector; 
    }

		public void clear () {
			vector.removeAllElements();
			stack.clear();
		}
		
	public void parse (String location) {
		try {
			URLConnection urlCon = new URL(location).openConnection();
			BufferedReader br = new BufferedReader(new InputStreamReader(urlCon.getInputStream()));
			String line,token,lastToken = null;
			
			//Alle Zeilen einlesen
			while ((line=br.readLine())!=null)
			{
				line = line.trim(); // delete leading or tail whitespaces
				StringTokenizer st = new StringTokenizer(line,"<>/",true);
				while (st.hasMoreTokens()) {
					token = st.nextToken();
					if (token.equals("<")) {
						token = st.nextToken();
						if (token.equals("/")) {
							endElement(st.nextToken());
						} else {
							startElement(token);
						}
					} else if (token.equals(">")) {
						// do nothing
					} else if (token.equals("/")) { // true or false
						endElement(lastToken);
					} else {
						characters(token);
					}
					lastToken=token;
				}
			}
		}
		catch(Exception e) {
			System.err.println(e.toString());
		}		
	}
	

    // ----- 

    public void startElement(String name) {
        isStackReadyForText = false;

        // if next element is complex, push a new instance on the stack
        // if element has attributes, set them in the new instance
		if (name.equals("ic")) {				
			stack.push(new Vector());
		} 
		else if (name.equals("a")) {				
			stack.push(new Vector());
		}
        else if (name.equals("r")) {
            stack.push(new StringBuffer());
            isStackReadyForText = true;
        }
        else if (name.equals("s")) {
            stack.push(new StringBuffer());
            isStackReadyForText = true;
        }
        else if (name.equals("i")) {
            stack.push(new StringBuffer());
            isStackReadyForText = true;
        }
        else  {
            // do nothing
        }                
    }

    // ----- 

    public void endElement(String name ) {
        isStackReadyForText = false;
        Object tmp = stack.pop();

        if (name.equals("a")) {
			if (stack.empty())
				stack.push(tmp); // main array, gets set in plist
			else
				((Vector)stack.peek()).add(tmp);
		}
        else if (name.equals("r")) {
			((Vector)stack.peek()).add(new Float(tmp.toString()));
        }
        else if (name.equals("i")) {
			((Vector)stack.peek()).add(new Integer(tmp.toString()));
        }
        else if (name.equals("s")) {
			((Vector)stack.peek()).add(tmp.toString());
        }
		else if (name.equals("t")) {
			((Vector)tmp).add(new Boolean(true));
			stack.push(tmp); // nothing gets pushed in startElement
        }
        else if (name.equals("f")) {
			((Vector)tmp).add(new Boolean(false));
			stack.push(tmp); // nothing gets pushed in startElement
        }
		
		else if (name.equals("ic")) {
			vector = (Vector) tmp;
        }
    }

    // -----
    
    public void characters(String data) {
        // if stack is not ready, data is not content of recognized element
        if (isStackReadyForText == true) {
            ((StringBuffer)stack.peek()).append(data);
        }
        else {
            // read data which is not part of recognized element
        }
    }
}