//
//  OCVLoader.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 

import java.awt.*;
import java.util.*;
import java.net.*;


public class OCVLoader implements Runnable {
	
    // Thread
	private Thread thread = null;
	private String path = null;
	private InfiniteCanvasApplet myApplet = null;
	private OCVDocument document = null;
	private MediaTracker imageTracker = null;

	public OCVLoader (InfiniteCanvasApplet applet) {
		myApplet = applet;
		imageTracker = new MediaTracker(myApplet);
		
		// start a new startup thread
		thread = new Thread(this);
		thread.start();
	}
	
	public void run() {
		if (thread == Thread.currentThread()) {
			// 1.step: start loading the xml file
			myApplet.setLoadingStatus(1);
			if (!loadDocument()) {
				myApplet.setLoadingStatus(-1);
				return;
			}
		
			// 2.step: add media to tracker
			if (!loadMedia()) {
				myApplet.setLoadingStatus(-2);
				return;
			}
			
			// 3. step: load panels (slots 0-8) 
			try {
				System.out.println("1");
				imageTracker.checkAll(true); // starts loading all images
				imageTracker.waitForID(0);
				myApplet.setLoadingStatus(2);
								System.out.println("2");

				imageTracker.waitForID(1);
				myApplet.setLoadingStatus(3);
								System.out.println("3");

				imageTracker.waitForID(2);
				myApplet.setLoadingStatus(4);
								System.out.println("4");

				imageTracker.waitForID(3);
				myApplet.setLoadingStatus(5);
				imageTracker.waitForID(4);
				myApplet.setLoadingStatus(6);
				imageTracker.waitForID(5);
				myApplet.setLoadingStatus(7);
				imageTracker.waitForID(6);
				myApplet.setLoadingStatus(8);
				imageTracker.waitForID(7);
				myApplet.setLoadingStatus(9);
				imageTracker.waitForID(8);
				myApplet.setLoadingStatus(10);
			} catch (InterruptedException e) {
				myApplet.setLoadingStatus(-3);
				System.err.println("Error: Couldn't load one or more panels!");
				return;
			}
			
			// 4. finish up
			document.finishLoading(myApplet);
			myApplet.startDisplayingDocument(document);
		}
	}	
	

	private boolean loadDocument () {
		Vector vector;
		OCVDocumentUnmarshaller handler;
		
		// extract document location from parameter
		path = myApplet.getParameter("document");
		if (path == null) {
			System.err.println("Error: Empty Path!");
			return false;
		}
		
		// create full path
		try {
			path = new URL(myApplet.getDocumentBase(),path).toString().concat("/");
		}
		catch (MalformedURLException e) {
			System.err.println("Error: Invalid Path!");
			return false;
		}
		
		// read and parse file
		try {
			handler = new OCVDocumentUnmarshaller();
			handler.parse(path.concat("comic.xml"));
			vector = handler.vector();
		}
		catch (Exception e) {
			System.err.println("Error: Couldn't parse xml file!");
			return false;
		}
		
		// create document structur from vector
		if (vector != null) {
			document = new OCVDocument();
			document.initWithVector(vector);
			vector.removeAllElements();
			vector = null;
			handler.clear();
			handler = null;
			return true; // normal exit point!
		} else {
			System.err.println("Error: Couldn't parse document!");
			return false; // vector == null
		}
	}

	private boolean loadMedia() {
		Image panelImage = null;
		String sourcePath = null;
		OCVPanelObject panelObject = null;
		int id = 0;
		
		try {
			//load panels
			sourcePath = path.concat("panels/");
			for (Enumeration e = document.canvasObjects().elements() ; e.hasMoreElements() ;) {
				OCVObject object = (OCVObject) e.nextElement();
				if (object.className().equals("OCVPanelObject")) { // oly panels and not trails
					panelObject = (OCVPanelObject) object;
					panelImage = myApplet.getImage(new URL(sourcePath.concat(panelObject.panelPath())));
					imageTracker.addImage(panelImage, id);
					panelObject.setImage(panelImage);

					// there are 9 loading slots for panels
					// the panels are equally partitioned
					id = (id+1) % 9;
				}
			}
			return true;
		}
		catch (MalformedURLException e) {
			System.err.println("Error: Wrong or missing path to panel/image!");
			return false;
		}
	}
}
