//
//  OCVNavFrameObject.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
import java.util.*;
import java.awt.geom.*;
import java.awt.*;

public class OCVNavFrameObject extends OCVRectObject {
	boolean isStartFrame;
	boolean waiting;
	Vector activeComponent;
	
    public OCVNavFrameObject() {
        className = "OCVNavFrameObject";
		activeComponent = new Vector();
    }
	
	public void initWithVector(Vector vector) {
		super.initWithVector(vector);
		isStartFrame = (((Boolean) vector.elementAt(7)).booleanValue());
		waiting = (((Boolean) vector.elementAt(8)).booleanValue());
		
		//activeComponent
		Vector objectsVector = ((Vector) vector.elementAt(9));
		int c = objectsVector.size();
		int i = 0;
		
		for (i=0; i<c; i++) {
			OCVActiveComponent component = new OCVActiveComponent();
			component.initWithVector((Vector)(objectsVector.elementAt(i)));
			component.setFrame(this);
			activeComponent.add(component);
		}
	}
	
	public boolean isStartFrame () {
		return isStartFrame;
	}
	
	public void setIsStartFrame(boolean flag) {
		isStartFrame = flag;
	}
	
	public void setWaiting(boolean flag) {
		waiting = flag;
	}
	
	public boolean waiting() {
		return waiting;
	}
	
	public boolean hasActiveComponent() {
		if (activeComponent.size() > 0)
			return true;
		else
			return false;
	}

	public OCVRectObject activeComponentHitTest(Point aPoint) {
		OCVRectObject object;
		int i = 0;
		
		aPoint.y = -aPoint.y;
		
		while (i < activeComponent.size())  {
			object = (OCVRectObject) activeComponent.elementAt(i);
			
			if (object.hitTest(aPoint)) {
				return object;
			}
			i++;
		}
		
		return null;
	}
	
	
	public Point convertPointToActiveComponent(Point aPoint) {
		try {
			transform.inverseTransform(aPoint,aPoint);
		}
		catch (NoninvertibleTransformException e) {
			System.err.println("EXCEPTION");
		}   
		aPoint.x +=  (int)rect.width/2.0;
		aPoint.y -=  (int)rect.height/2.0;
		
		return aPoint;
	}
	
	public OCVRectObject activeComponentFromID(int id) {
		OCVRectObject object;
		int i = 0;
		int c = activeComponent.size();
		while (i < c) {
			object = (OCVRectObject) activeComponent.elementAt(i);
			if (object.rectID() == id)
				return object;
			i++;
		}
		return null;
	}
}


