//
//  OCVNavTrailObject.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
import java.util.*;
import java.awt.geom.*;

public class OCVNavTrailObject extends OCVPathObject {
	int transition;
	int speed;
	
	public OCVNavTrailObject() {
        className = "OCVNavTrailObject";
    }
	
	public void initWithVector(Vector vector) {
		super.initWithVector(vector);
		
		startRectID = ((Integer) vector.elementAt(2)).intValue();
		endRectID = ((Integer) vector.elementAt(3)).intValue();
		transition = (((Integer) vector.elementAt(4)).intValue());
		speed = (((Integer) vector.elementAt(5)).intValue());
	}
	
    public void setTransition(int newType) {
		transition = newType;
	}
	
	public int transition() {
		return transition;
	}
	
	public void setSpeed(int newSpeed) {
		speed = newSpeed;
	}
	
	public int speed() {
		return speed;
	}
	
	// Navigation
	public int segments() {
		int count=0;
		PathIterator pi = path.getPathIterator(null);
		while (!pi.isDone()) {
			count++;
			pi.next();
		}
		
		return count-1; // we don't need moveTo
	}
	
	public Point2D.Double endPointOfSegment(int segment) {
		Point2D.Double returnPoint = new Point2D.Double(0,0);
		
		int count = 0;
		PathIterator pi = path.getPathIterator(null);
		double[] buffer = new double[6];
		while (!(pi.isDone())){
			if (count == segment) {
				int type = pi.currentSegment(buffer);
				if (type == PathIterator.SEG_MOVETO) {
					returnPoint = new Point2D.Double(buffer[0], buffer[1]);
				} else if (type == PathIterator.SEG_LINETO) {
					returnPoint = new Point2D.Double(buffer[0], buffer[1]);
				}
				break;
			} else {
				count++;
			}
			pi.next();
		}
		
		return returnPoint;
	}
	
	public int length(OCVDocument document) {
		int count = segments()+1;
		int i = 1;
		int length = 0;
		
		while (i < count) {
			length += lengthOfSegment(i, document);
			i++;
		}
		return length;
	}
	
	public double lengthOfSegment(int segment, OCVDocument document) {
		Point2D.Double startPoint;
		OCVRectObject startRect = document.rectObjectFromID(startRectID);
		if ((segment-1 == 0) && (startRect.isActiveComponent()))
			startPoint = ((OCVActiveComponent)startRect).frame().center();
		else
			startPoint = endPointOfSegment(segment-1);
		
		Point2D.Double endPoint = endPointOfSegment(segment);
		double width = endPoint.x - startPoint.x;
		double height = endPoint.y - startPoint.y;
		
		return Math.abs(Math.sqrt(width*width+height*height)); //pythagoras
	}
	
}
