//
//  OCVPanelObject.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
import java.awt.*;
import java.awt.image.*;
import java.lang.Math;
import java.util.*;

public class OCVPanelObject extends OCVRectObject {
	Image image = null;
	Color borderColor;
	float borderThickness;
	float transparency;
	int style;
	int panelID;
	String panelPath;
	Rectangle borderRect = null;
    
	public OCVPanelObject() {
		className = "OCVPanelObject";
	}
	
	public void initWithVector(Vector vector) {
		super.initWithVector(vector);
		panelPath = ((String) vector.elementAt(7));
		style = ((Integer) vector.elementAt(8)).intValue();
		transparency = ((Float) vector.elementAt(9)).floatValue();
		setBorderColor(((Float)vector.elementAt(10)).floatValue(), ((Float)vector.elementAt(11)).floatValue(), 
					   ((Float)vector.elementAt(12)).floatValue());
		borderThickness = ((Float) vector.elementAt(13)).floatValue();
		if (borderThickness != 0) { // increase bounds so also the border is included
			int thickness = (int) Math.ceil(borderThickness);
			bounds.x -= thickness;
			bounds.y -= thickness;
			bounds.width += 2 * thickness;
			bounds.height += 2 * thickness;
		}
	}
		
    public void setImage(Image aImage) {
        image = aImage;
	}
	
	public void finishLoading(InfiniteCanvasApplet applet) {
		
		// create drawing image
		if (style == 0)  { // scale
			BufferedImage tmpImage =  new BufferedImage((int)rect.width,(int)rect.height,BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2 = (Graphics2D) tmpImage.createGraphics();
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);		
			g2.drawImage(image,0,0,(int)rect.width,(int)rect.height,null);   // drawImage
			image.flush();
			image = tmpImage;
		} else if (style == 1) { // crop
			int cropWidth =  (rect.width > image.getWidth(null) ? image.getWidth(null) : (int)rect.width);
			int cropHeight = (rect.height > image.getHeight(null) ? image.getHeight(null) : (int)rect.height);
			
			CropImageFilter cropFilter = new CropImageFilter(0, 0, cropWidth, cropHeight);
			Image tmpImage = applet.createImage(new FilteredImageSource(image.getSource(), cropFilter));
			image.flush();
			image = tmpImage;
		} else { // tile
			int x,y;
			int imageHeight = image.getHeight(null);
			int imageWidth = image.getWidth(null);
			BufferedImage tmpImage =  new BufferedImage((int)rect.width,(int)rect.height,BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2 = (Graphics2D) tmpImage.createGraphics();
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			
			x = 0;
			while (x < rect.width) {
				y = 0;
				while (y < rect.height) {
					g2.drawImage(image,x,y,null);   // drawImage
					y += imageHeight;
				}
				x += imageWidth;
			}
			
			image.flush();
			image = tmpImage;
		}
		
		// create borderRect
		if (borderThickness != 0.0) {
			borderRect = new Rectangle(-(int)(rect.width/2),-(int)(rect.height/2),(int)rect.width,(int)rect.height);
			int offset = (int) Math.floor(borderThickness/2.0);
			borderRect.x -= offset;
			borderRect.y -= offset;
			offset *= 2;
			borderRect.width  += offset;
			borderRect.height += offset;
		}
	}
    
    public void setBorderColor(float r, float g, float b) {
        borderColor = new Color(r,g,b);
    }
    
    public void setTransparency(float newTransparency) {
        transparency = newTransparency;
    }
    
    public void setStyle(int newStyle) {
        style = newStyle;
    }
	
	public int style() {
        return style;
    }	
    
    public void setPanelID(int newID) {
        panelID = newID;
    }
    
    public int panelID() {
        return panelID;
    }
	
	public String panelPath () {
		return panelPath;
	}

    public boolean drawObject(Graphics2D g2, float fraction) {
		// apply transform
		g2.transform(transform);
		
		// apply fraction
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,transparency * fraction));
		
		// draw image
		g2.drawImage(image,-(int)(rect.width/2),-(int)(rect.height/2),null);
		
		if ((borderThickness != 0.0) && (borderRect != null)) {
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,fraction));
			g2.setStroke(new BasicStroke(borderThickness));
			g2.setColor(borderColor);
			g2.draw(borderRect);
		}
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,1));
		
		return true;
    }
}
