//
//  OCVRectObject.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
import java.awt.geom.*;
import java.awt.*;
import java.util.*;

public class OCVRectObject extends OCVObject {
    Rectangle2D.Double rect;
    float rotation;
	int rectID;
	AffineTransform transform;
	
    public OCVRectObject() {
        className = "OCVRectObject";
        rect = new Rectangle2D.Double(0,0,0,0);
		rectID=0;
    }
	
	public void initWithVector(Vector vector) {
		rect = new Rectangle2D.Double(((Float)vector.elementAt(1)).doubleValue(), 
									  -((Float)vector.elementAt(2)).doubleValue(),
									  ((Float)vector.elementAt(3)).doubleValue(), 
									  ((Float)vector.elementAt(4)).doubleValue());
		rotation = (float) (-OCVMath.toRadians(((Float) vector.elementAt(5)).floatValue()));
		rectID = ((Integer) vector.elementAt(6)).intValue();
		
		// transformation matrix
		transform = new AffineTransform();
		transform.translate(rect.x,rect.y);
		transform.rotate(rotation);
		
		// set bounds
		Rectangle2D.Double drawRect = new Rectangle2D.Double(-rect.width/2,-rect.height/2,rect.width,rect.height);

		GeneralPath path = new GeneralPath(drawRect);
		path.transform(transform);
		
		bounds = (Rectangle2D.Float)path.getBounds2D();
	}
	
	public boolean hitTest(Point aPoint) {
		if (rect.contains(aPoint)) {
			return true;
		}
		return false;
	}
		
	public float rotation() {
		return rotation;
	}
	
	public int rectID() {
		return rectID;
	}
	
	public Point2D.Double center() {
		return new Point2D.Double(bounds.getCenterX(),bounds.getCenterY());
	}
	
	public double width() {
		return rect.width;
	}
	
	public double height() {
		return rect.height;
	}
	
	public boolean isActiveComponent() {
		return false;
	}
}
