//
//  OCVTrailObject.java
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
import java.awt.*;
import java.util.*;
import java.awt.geom.*;

public class OCVTrailObject extends OCVPathObject {
    Rectangle2D.Float bounds;
	BasicStroke basicStroke, trailBasicStroke;
	AffineTransform arrowTransform1;
	AffineTransform arrowTransform2;
	GeneralPath arrowPath;
    
    public OCVTrailObject() {
        className = "OCVTrailObject";
    }
    
	public void initWithVector(Vector vector) {
		super.initWithVector(vector);
		
		setColor(((Float)vector.elementAt(2)).floatValue(), ((Float)vector.elementAt(3)).floatValue(), 
				 ((Float)vector.elementAt(4)).floatValue());
		arrow = ((Integer) vector.elementAt(6)).intValue();
		
		
		// create basis stroke
		float thickness = ((Float)vector.elementAt(5)).floatValue();
		int dashStyle = ((Integer) vector.elementAt(7)).intValue();
		basicStroke = new BasicStroke(thickness, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND, 10.0f, createDashArray(dashStyle), 0);
		trailBasicStroke = new BasicStroke(thickness, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND, 10.0f, null, 0);

		// create ArrowTransform1
		if ((arrow == 1) || (arrow == 3) || (arrow == 4) || (arrow == 6)) {
			float arrowRotation1 = ((Float) vector.elementAt(8)).floatValue();
			Point2D.Float endPoint = (Point2D.Float) path.getCurrentPoint();
			arrowTransform1 = new AffineTransform();
			arrowTransform1.translate(endPoint.x,endPoint.y);
			arrowTransform1.rotate(-OCVMath.toRadians(arrowRotation1));
			
		}
		
		// create ArrowTransform2
		if ((arrow == 2) || (arrow == 3) || (arrow == 5) || (arrow == 6)) {
			float arrowRotation2 = ((Float) vector.elementAt(9)).floatValue();
			Vector objectsVector = ((Vector) vector.elementAt(1));
			Point2D.Float startPoint = new Point2D.Float(((Float) objectsVector.elementAt(1)).floatValue(),
														 -((Float) objectsVector.elementAt(2)).floatValue());
			arrowTransform2 = new AffineTransform();
			arrowTransform2.translate(startPoint.x,startPoint.y);
			arrowTransform2.rotate(-OCVMath.toRadians(arrowRotation2));
		}
		
		// create Arrow
		if ((arrow <= 3)) {
			arrowPath = new GeneralPath();
			arrowPath.moveTo(-10,10);
			arrowPath.lineTo(0,0);
			arrowPath.lineTo(-10,-10);
		} else {
			arrowPath = new GeneralPath();
			arrowPath.moveTo(-15,7);
			arrowPath.lineTo(0,0);
			arrowPath.lineTo(-15,-7);
			arrowPath.moveTo(-15,7);
			arrowPath.lineTo(-5,0);
			arrowPath.lineTo(-15,-7);			
		}
	}
	
    public boolean drawObject(Graphics2D g2, float fraction) {
        g2.setStroke(basicStroke);
        g2.setColor(color);
       
        g2.draw(path);
		
		if (arrow != 0) {
			AffineTransform saveAT = g2.getTransform();
			g2.setStroke(trailBasicStroke);
			if (arrow < 4) {
				if ((arrow == 1) || (arrow == 3)) {
					g2.transform(arrowTransform1);
					g2.draw(arrowPath);
					g2.setTransform(saveAT);
				}
				if ((arrow == 2) || (arrow == 3)) {
					g2.transform(arrowTransform2);
					g2.draw(arrowPath);
					g2.setTransform(saveAT);
				}
			} else {
				if ((arrow == 4) || (arrow == 6)) {
					g2.transform(arrowTransform1);
					g2.draw(arrowPath);
					g2.setTransform(saveAT);
				}
				if ((arrow == 5) || (arrow == 6)) {
					g2.transform(arrowTransform2);
					g2.draw(arrowPath);
					g2.setTransform(saveAT);
				}
			}
		}
		return true;
    }
    
    public float[] createDashArray(int style) {
		float[] dashArray;
        switch (style) {
			case 1:
                dashArray = new float[2];
                dashArray[0] = 5;
                dashArray[1] = 5;
                break;
            case 2:
                dashArray = new float[3];
                dashArray[0] = 8;
                dashArray[1] = 3;
                dashArray[2] = 8;
                break;
            case 3:
                dashArray = new float[3];
                dashArray[0] = 9;
                dashArray[1] = 6;
                dashArray[2] = 3;
                break;
			default: // no dash
                dashArray = null;
                break;
        }
		return dashArray;
    }
}
